<?php

namespace Params\Params\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Params\Params\Http\Requests\ParamsRequest;
use Params\Params\Interfaces\ParamsRepositoryInterface;
use Params\Params\Models\Params;

/**
 * Resource controller class for params.
 */
class ParamsResourceController extends BaseController
{

    /**
     * Initialize params resource controller.
     *
     * @param type ParamsRepositoryInterface $params
     *
     * @return null
     */
    public function __construct(ParamsRepositoryInterface $params)
    {
        parent::__construct();
        $this->repository = $params;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Params\Params\Repositories\Criteria\ParamsResourceCriteria::class);
    }

    /**
     * Display a list of params.
     *
     * @return Response
     */
    public function index(ParamsRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Params\Params\Repositories\Presenter\ParamsPresenter::class)
                ->$function();
        }

        $params = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('params::params.names'))
            ->view('params::params.index', true)
            ->data(compact('params', 'view'))
            ->output();
    }

    /**
     * Display params.
     *
     * @param Request $request
     * @param Model   $params
     *
     * @return Response
     */
    public function show(ParamsRequest $request, Params $params)
    {

        if ($params->exists) {
            $view = 'params::params.show';
        } else {
            $view = 'params::params.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('params::params.name'))
            ->data(compact('params'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new params.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ParamsRequest $request)
    {

        $params = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('params::params.name')) 
            ->view('params::params.create', true) 
            ->data(compact('params'))
            ->output();
    }

    /**
     * Create new params.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ParamsRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $params                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('params::params.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('params/params/' . $params->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/params/params'))
                ->redirect();
        }

    }

    /**
     * Show params for editing.
     *
     * @param Request $request
     * @param Model   $params
     *
     * @return Response
     */
    public function edit(ParamsRequest $request, Params $params)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('params::params.name'))
            ->view('params::params.edit', true)
            ->data(compact('params'))
            ->output();
    }

    /**
     * Update the params.
     *
     * @param Request $request
     * @param Model   $params
     *
     * @return Response
     */
    public function update(ParamsRequest $request, Params $params)
    {
        try {
            $attributes = $request->all();

            $params->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('params::params.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('params/params/' . $params->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('params/params/' . $params->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the params.
     *
     * @param Model   $params
     *
     * @return Response
     */
    public function destroy(ParamsRequest $request, Params $params)
    {
        try {

            $params->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('params::params.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('params/params/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('params/params/' . $params->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple params.
     *
     * @param Model   $params
     *
     * @return Response
     */
    public function delete(ParamsRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('params::params.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('params/params'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/params/params'))
                ->redirect();
        }

    }

    /**
     * Restore deleted params.
     *
     * @param Model   $params
     *
     * @return Response
     */
    public function restore(ParamsRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('params::params.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/params/params'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/params/params/'))
                ->redirect();
        }

    }

}
